/* Utility program to send commands and data to a H-P LaserJet or compatible */
/* printer connected to Centronics parallel port LPTn: for some value of n   */

/* Copyright 1988 by Daniel Ross                                             */

/* Coded in Microsoft C 5.1 by Daniel Ross, starting 10/31/88                */
/* Revised 11/2/88   13:15                                                   */

/* This program sends commands and files of data to the printer.  The files  */
/* may contain either fonts, or text to be printed.                          */

/* Input to this program is a command file of ASCII text.  The text is       */
/* organized into records, each record ending with CarriageReturn and        */
/* LineFeed characters.                                                      */

/* Each record may specify a single printer command (possibly followed by a  */
/* comment), or the record may name a file to be sent to the printer, or the */
/* record may contain only a comment.                                        */

/* Any printing characters at the start of a record are significant.  A      */
/* nonprinting character, such as Space or Tab, terminates the significant   */
/* part of the record.  The remainder of the record is treated as a comment. */
/* Any record that starts with a nonprinting character, is treated as        */
/* comment only.                                                             */

/* If the 1st character in the record is a MinusSign, the printing           */
/* characters that follow must name a file (including path, if needed).      */
/* The file name is terminated by any nonprinting character.  This program   */
/* sends the contents of the file to the printer.                            */

/* If the 1st character in the record is any printing character except       */
/* MinusSign, the record is treated as a command to the printer.  This       */
/* program sends to the printer an Escape character followed by the          */
/* significant printing characters in the record.  The command is terminated */
/* by any nonprinting character.                                             */

/* If your printer is not connected to LPT1:, change the definition of       */
/* PrinterNr (below).                                                        */

/* Thanks to David Heintz, Software Consultant, Santa Cruz, CA, for          */
/* debugging help.                                                           */

#include <STDLIB.H>
#include <STDIO.H>
#include <CONIO.H>
#include <DOS.H>

#define FALSE               0
#define TRUE                1
#define TorF                int

#define Escape              0x1B

#define PrinterIrp          0x17
#define PrinterNr           0           /* This selects LPT1:                */

/* int86 printer commands                                                    */
#define InitPrinterPort     1
#define WriteToPrinter      0
#define ReadPrinterStatus   2

/* int86 printer status bits                                                 */
#define Timeout             0x01
#define IOErr               0x08
#define Selected            0x10
#define OutOfPaper          0x20
#define PrinterAck          0x40
#define PrinterReady        0x80

char    CmdBuf [256] ;                  /* Buffer for 1 record from the      */
                                        /* command file                      */
char        * CmdFileName ;
FILE        * CmdFileHandle ;
char        * DataFileName   =   & CmdBuf [1] ;
FILE        * DataFileHandle ;
char        * Cursor ;
int         DataChar ;
TorF        SendingFromDataFile ;
long        DataByteNr ;
union REGS  CallRegs, RetRegs ;

int   PrintingChar (int GivenByte)
  {
    return ((GivenByte > ' ')   &&   (GivenByte <= '~')) ;
  }

void   Send1ByteToPrinter (int GivenByte)
  {
    CallRegs.x.dx   =   PrinterNr ;
    CallRegs.h.ah   =   WriteToPrinter ;
    CallRegs.h.al   =   (char) GivenByte ;
    for (;;)
      {
        int86 (PrinterIrp, & CallRegs, & RetRegs) ;
        if     (RetRegs.h.ah   &   Timeout)                   continue ;
        if ( ! (RetRegs.h.ah   &   (IOErr | OutOfPaper)))     break ;
        printf ("\n\nError writing to the printer") ;
        if (SendingFromDataFile)
          printf
            (
              ",\n   data byte number %ld(decimal) = %lx(hex)"
                ,
              DataByteNr
                ,
              DataByteNr
            ) ;
        printf (":  data value = %02x(hex)", GivenByte) ;
        printf ("\nPrinter status = %02x(hex)", ((int) RetRegs.h.ah) & 0xFF) ;
        break ;
      } ;
  }

int   main (int argc, char * argv [], char * envp [])
  {
    if (argc != 2)
      {
        printf ("\nExactly 1 parameter needed = name of command file") ;
        exit (1) ;
      } ;
    CmdFileName   =   argv [1] ;
    if ( ! (CmdFileHandle = fopen (CmdFileName, "rt")))
      {
        printf ("\nUnable to open command file %s", CmdFileName) ;
        exit (1) ;
      } ;
    CallRegs.x.dx   =   PrinterNr ;
    CallRegs.h.ah   =   InitPrinterPort ;
    CallRegs.h.al   =   0 ;
    int86 (PrinterIrp, & CallRegs, & RetRegs) ;
    for (;;)
      {
        if ( ! fgets (CmdBuf, 256, CmdFileHandle))
          {
            if (feof (CmdFileHandle))   break ;
            if (ferror (CmdFileHandle))
              {
                printf ("\nError reading command file %s", CmdFileName) ;
                exit (1) ;
              } ;
          } ;
        printf ("%s", CmdBuf) ;         /* Show the record                   */
        Cursor = CmdBuf ;
        if ( ! PrintingChar ( * Cursor))   continue ;   /* Comment only      */
        if ( * Cursor   ==   '-')
          {
            /* The record names a file to send to the printer.  Terminate    */
            /* the record after the file name, thereby effectively deleting  */
            /* any comments.                                                 */
            while (PrintingChar (((int) ( * Cursor ++ )) & 0xFF))   ;
            * Cursor   =   0 ;
            if ( ! (DataFileHandle = fopen (DataFileName, "rb")))
              {
                printf ("\nUnable to open data file %s", DataFileName) ;
                exit (1) ;
              } ;
            SendingFromDataFile = TRUE ;
            for (DataByteNr = 0 ;   ;   ++ DataByteNr)
              {
                DataChar   =   fgetc (DataFileHandle) ;
                if (DataChar == EOF)
                  {
                    if (feof (DataFileHandle))   break ;
                    if (ferror (DataFileHandle))
                      {
                        printf ("\nError reading data file %s", DataFileName) ;
                        exit (1) ;
                      } ;
                  } ;
                Send1ByteToPrinter (DataChar & 0xFF) ;
              } ;
            if (fclose (DataFileHandle))
              {
                printf ("\nError closing data file %s", DataFileName) ;
                exit (1) ;
              } ;
          }
        else
          {
            /* The record contains a printer command                         */
            SendingFromDataFile = FALSE ;
            Send1ByteToPrinter (Escape) ;
            for (;;)
              {
                DataChar   =   ((int) ( * Cursor ++ )) & 0xFF ;
                if ( ! PrintingChar (DataChar))   break ;
                Send1ByteToPrinter (DataChar) ;
              } ;
          } ;
      } ;
    if (fclose (CmdFileHandle))
      {
        printf ("\nError closing command file %s", CmdFileName) ;
        exit (1) ;
      } ;
    exit (0) ;
  }

/* End of file                                                               */

